"""A collection of general purpose utility functions and classes that can be 
used within scripts and web applications.

Copyright (c) 2017, Hazeltek Solutions.
"""

__author__  = 'Hazeltek Solutions'
__version__ = '0.1'


# expose significant functions and classes defined within the core module
from .core import AttrDict, Coordinates, CoordinatesLite
