import os
import elixr
# from distutils.core import setup
from setuptools import setup

setup(
  name = 'elixr',
  packages = ['elixr'], # this must be the same as the name above
  version = '0.1',
  description = 'A python general purpose utility library',
  author = 'Abdul-Hakeem Shaibu',
  author_email = 'info@hazeltek.com',
  url = 'https://github.com/babdulhakim2/elixr.git', # use the URL to the github repo
  download_url = 'https://github.com/babdulhakim2/elixr.git', # I'll explain this in a second
  keywords = ['testing', 'logging', 'example'], # arbitrary keywords
  classifiers = [],
)